library(tidyverse)
library(gridExtra)
library(grid)
library(Seurat)

if (!requireNamespace("remotes", quietly = TRUE)) {
    install.packages("remotes")
}
if (!require(SeuratDisk)){
    remotes::install_github("mojaveazure/seurat-disk")
}
library(SeuratDisk)

results_folder<-file.path("results","OUT")
data_folder<-file.path(results_folder,"COUNT","AGGR_baseline_transcriptome")
out_folder<-file.path(results_folder,"ANALYSIS_SCVI")
matrix_filename<-"filtered_feature_bc_matrix.h5"
qc_categories_filename<-"QC_categories.csv"

analysis_name<-"skin"
analysis_htos<-c("HTO-Skin_NL","HTO-Skin_L")

output_prefix<-sprintf("baseline_transcriptome_%s.SCT",analysis_name)
output_rds<-sprintf("%s.rds",output_prefix)
output_h5seurat<-sprintf("%s.counts.h5Seurat",output_prefix)

print(output_prefix)

print("READING DATA")
data<-Read10X_h5(file.path(data_folder,matrix_filename), use.names = TRUE,unique.features =FALSE)

#SELECT ONLY PASS QC CELLS
print(sprintf("SELECTING PASS QC CELLS IN THE %s SUBSET",analysis_name))
meta.data<-read.csv(file.path(data_folder,qc_categories_filename)) %>% filter(pass_QC=="True")  %>% filter(hto_demux %in% analysis_htos)
rownames(meta.data)<-meta.data[['barcode']]

#DEFINE BATCH
meta.data<-meta.data %>% extract(sample, "batch", "10x_CITE_SrcX_(P[0-9]+_V[0-9]+)_.*")

data<-data[,rownames(meta.data)]

#USE ENSEMBL IDS
print("SETTING ENSEMBL ID AS GENE IDENTIFIER")
rownames(data)<-read.csv(file.path(data_folder,"filtered_feature_bc_matrix","features.tsv.gz"),header=FALSE,sep='\t')[['V1']]

sobj <- CreateSeuratObject(counts = data, meta.data =meta.data)
rm(data)
gc()

print("CALCULATING TECHNICAL CONFOUNDERS")
sobj <- PercentageFeatureSet(sobj, pattern = "^MT-", col.name = "percent.mt")
sobj <- PercentageFeatureSet(sobj, pattern = "^RP[SL]-", col.name = "percent.ribo")

print("RUNNING SCT TRANSFORMATION")
sobj <- SCTransform(sobj, 
                    method = "glmGamPoi",
                    vars.to.regress = c("percent.mt","percent.ribo"), 
                    verbose = TRUE,
                    return.only.var.genes=FALSE)
#SAVE RDS
print("SAVING RDS FILE")
saveRDS(sobj, file.path(out_folder,output_rds))

#CONVERT TO H5AD ONLY COUNTS
sobj_norm<-CreateSeuratObject(sobj[['SCT']]@counts)

print("SAVING H5SEURAT FILE")
SaveH5Seurat(sobj_norm, filename = file.path(out_folder,output_h5seurat),overwrite = TRUE)

print("SAVING H5AD FILE")
Convert(
  file.path(out_folder,output_h5seurat),
  dest = "h5ad",
  assay = "RNA",
  overwrite = TRUE,
  verbose = TRUE
)

print("REMOVING H5SEURAT FILE")
file.remove(file.path(out_folder,output_h5seurat))